@@ -2,24 +2,35 @@
 /*
  * ghex-window.c: a ghex window
  *
- * Copyright (C) 2002 the Free Software Foundation
+ * Copyright (C) 2002 - 2004 the Free Software Foundation
  *
  * Author: Jaka Mocnik  <jaka@gnu.org>
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif /* HAVE_CONFIG_H */
 
 #include <gnome.h>
+
 #include <bonobo-activation/bonobo-activation.h>
+
 #include <libgnomeui/gnome-window-icon.h>
+
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
 
+#include <libgnomevfs/gnome-vfs-ops.h>
+
 #include <math.h>
 #include <ctype.h>
 
 #include "ghex-window.h"
-#include "ghex.h"
+#include "ui.h"
+#include "converter.h"
+#include "chartable.h"
+#include "configuration.h"
+#include "hex-dialog.h"
 
 #define GHEX_WINDOW_DEFAULT_WIDTH 320
 #define GHEX_WINDOW_DEFAULT_HEIGHT 256
@@ -70,16 +81,42 @@ ghex_window_drag_data_received(GtkWidget *widget,
         else
             newwin = NULL;
         while(*uri) {
-            if(!g_ascii_strncasecmp("file:", *uri, 5)) {
-                if(newwin == NULL) {
-                    newwin = ghex_window_new_from_file((*uri) + 5);
+            GError *err = NULL;
+            gchar *filename = g_filename_from_uri(*uri, NULL, &err);
+
+            uri++;
+            if(filename == NULL) {
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Can not open URI:\n%s"),
+                                             err->message);
+                g_error_free(err);
+                continue;
+            }
+
+	    	if(newwin == NULL)
+                newwin = ghex_window_new();
+            if(ghex_window_load(GHEX_WINDOW(newwin), filename)) {
+                if(newwin != GTK_WIDGET(win))
                     gtk_widget_show(newwin);
-                }
-                else
-                    ghex_window_load(newwin, (*uri) + 5);
                 newwin = NULL;
             }
-            uri++;
+            else {
+                GtkWidget *dlg;
+                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_OK,
+                                             _("Can not open file:\n%s"),
+                                             *filename);
+                gtk_widget_show(dlg);
+                gtk_dialog_run(GTK_DIALOG(dlg));
+                gtk_widget_destroy(dlg);
+            }
+            g_free(filename);
         }
         g_strfreev(win->uris_to_open);
         win->uris_to_open = NULL;
@@ -94,14 +131,14 @@ ghex_window_close(GHexWindow *win)
 
 	if(win->gh == NULL) {
         gtk_widget_destroy(GTK_WIDGET(win));
-		return;
+		return FALSE;;
 	}
 
 	doc = win->gh->document;
 	
 	if(doc->views->next == NULL) {
 		if(!ghex_window_ok_to_close(win))
-			return;
+			return FALSE;
 	}	
 
     /* We dont have to unref the document if the view is the only one */
@@ -127,6 +164,8 @@ ghex_window_close(GHexWindow *win)
 
     if (doc->views == NULL) /* If we have destroyed the last view */
       g_object_unref (G_OBJECT (doc));
+
+    return TRUE;
 }
 
 static gboolean 
@@ -147,7 +186,6 @@ ghex_window_set_sensitivity (GHexWindow *win)
 {
     BonoboUIComponent *uic = win->uic;
     gboolean allmenus = (win->gh != NULL);
-    gboolean anymenus = (ghex_window_get_list() != NULL);
 
     win->undo_sens = (allmenus && (win->gh->document->undo_top != NULL));
     win->redo_sens = (allmenus && (win->gh->document->undo_stack != NULL && win->gh->document->undo_top != win->gh->document->undo_stack));
@@ -173,6 +211,8 @@ ghex_window_set_sensitivity (GHexWindow *win)
                                   allmenus?"1":"0", NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/Replace", "sensitive",
                                   allmenus?"1":"0", NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/AdvancedFind", "sensitive",
+                                  allmenus?"1":"0", NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/GoToByte", "sensitive",
                                   allmenus?"1":"0", NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/InsertMode", "sensitive",
@@ -257,7 +297,7 @@ ghex_window_destroy(GtkObject *object)
             active_window = GHEX_WINDOW(window_list->data);
 
         if(GTK_OBJECT_CLASS(parent_class)->destroy)
-                GTK_OBJECT_CLASS(parent_class)->destroy(object);
+            GTK_OBJECT_CLASS(parent_class)->destroy(object);
 }
 
 static gboolean
@@ -358,7 +398,6 @@ ghex_window_listener (BonoboUIComponent           *uic,
                       gpointer                     user_data)
 {
 	GHexWindow *win;
-	GtkWidget *view;
 
 	g_return_if_fail (user_data != NULL);
 	g_return_if_fail (GHEX_IS_WINDOW (user_data));
@@ -379,7 +418,7 @@ ghex_window_listener (BonoboUIComponent           *uic,
 
         if(state && atoi(state)) {
             if(!GTK_WIDGET_VISIBLE(converter->window)) {
-                gtk_window_position(GTK_WINDOW(converter->window), GTK_WIN_POS_MOUSE);
+                gtk_window_set_position(GTK_WINDOW(converter->window), GTK_WIN_POS_MOUSE);
                 gtk_widget_show(converter->window);
             }
             raise_and_focus_widget(converter->window);
@@ -402,7 +441,7 @@ ghex_window_listener (BonoboUIComponent           *uic,
 
         if(state && atoi(state)) {
             if(!GTK_WIDGET_VISIBLE(char_table)) {
-                gtk_window_position(GTK_WINDOW(char_table), GTK_WIN_POS_MOUSE);
+                gtk_window_set_position(GTK_WINDOW(char_table), GTK_WIN_POS_MOUSE);
                 gtk_widget_show(char_table);
             }
             raise_and_focus_widget(char_table);
@@ -450,7 +489,6 @@ ghex_window_new(void)
     GHexWindow *win;
 	BonoboUIContainer *ui_container;
 	BonoboUIComponent *uic;
-    CORBA_Environment ev;
     const GList *doc_list;
 
 	static const GtkTargetEntry drag_types[] = {
@@ -590,6 +628,39 @@ ghex_window_sync_group_type(GHexWindow *win)
     bonobo_ui_component_set_prop(win->uic, group_path, "state", "1", NULL);
 }
 
+void
+ghex_window_update_status_message(GHexWindow *win)
+{
+#define FMT_LEN    128
+#define STATUS_LEN 128
+
+    gchar fmt[FMT_LEN], status[STATUS_LEN];
+    gint current_pos;
+    gint ss, se, len;
+
+    if(NULL == win->gh) {
+        ghex_window_show_status(win, " ");
+        return;
+    }
+
+    current_pos = gtk_hex_get_cursor(win->gh);
+    if(g_snprintf(fmt, FMT_LEN, _("Offset: %s"), offset_fmt) < FMT_LEN) {
+        g_snprintf(status, STATUS_LEN, fmt, current_pos);
+        if(gtk_hex_get_selection(win->gh, &ss, &se)) {
+            if(g_snprintf(fmt, FMT_LEN, _("; %s bytes from %s to %s selected"),
+                          offset_fmt, offset_fmt, offset_fmt) < FMT_LEN) {
+                len = strlen(status);
+                if(len < STATUS_LEN) {
+                    g_snprintf(status + len, STATUS_LEN - len, fmt, se - ss, ss, se);
+                }
+            }
+        }
+        ghex_window_show_status(win, status);
+    }
+    else
+        ghex_window_show_status(win, " ");
+}
+
 static void
 cursor_moved_cb(GtkHex *gtkhex, gpointer user_data)
 {
@@ -600,17 +671,10 @@ cursor_moved_cb(GtkHex *gtkhex, gpointer user_data)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
     current_pos = gtk_hex_get_cursor(gtkhex);
-
-	if((format = g_strdup_printf(_("Offset: %s"), offset_fmt)) != NULL) {
-		if((cursor_pos = g_strdup_printf(format, current_pos)) != NULL) {
-			ghex_window_show_status(win, cursor_pos);
-            g_free(cursor_pos);
-		}
-		g_free(format);
-	}
+    ghex_window_update_status_message(win);
     for (i = 0; i < 8; i++)
     {
-        /* returns  0 on buffer overflow, which is what we want */
+        /* returns 0 on buffer overflow, which is what we want */
         val.v[i] = gtk_hex_get_byte(gtkhex, current_pos+i);
     }
     hex_dialog_updateview(win->dialog, &val);
@@ -640,7 +704,7 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     else
         full_path = g_strdup(filename);
     
-    doc = hex_document_new(full_path);
+    doc = hex_document_new_from_file(full_path);
     g_free(full_path);
     if(!doc)
         return FALSE;
@@ -649,17 +713,24 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     gtk_hex_show_offsets(GTK_HEX(gh), show_offsets_column);
     g_signal_connect(G_OBJECT(doc), "document_changed",
                      G_CALLBACK(ghex_window_doc_changed), win);
+    g_signal_connect(G_OBJECT(doc), "undo",
+                     G_CALLBACK(set_doc_menu_sensitivity), win);;
+    g_signal_connect(G_OBJECT(doc), "redo",
+                     G_CALLBACK(set_doc_menu_sensitivity), win);;
+    g_signal_connect(G_OBJECT(doc), "undo_stack_forget",
+                     G_CALLBACK(set_doc_menu_sensitivity), win);;
     g_signal_connect(G_OBJECT(gh), "cursor_moved",
                      G_CALLBACK(cursor_moved_cb), win);
     gtk_widget_show(gh);
 
     vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(win), GNOME_PAD_SMALL);
     gtk_widget_show(vbox);
-    gtk_box_pack_start(GTK_BOX(vbox), gh, TRUE, TRUE, GNOME_PAD);
+    gtk_box_pack_start(GTK_BOX(vbox), gh, TRUE, TRUE, GNOME_PAD_SMALL);
 
     win->dialog = hex_dialog_new();
     win->dialog_widget = hex_dialog_getview(win->dialog);
-    gtk_box_pack_start(GTK_BOX(vbox), win->dialog_widget, FALSE, FALSE, GNOME_PAD);
+    gtk_box_pack_start(GTK_BOX(vbox), win->dialog_widget, FALSE, FALSE, GNOME_PAD_SMALL);
     state = bonobo_ui_component_get_prop (win->uic, "/commands/TypeDialog", "state", NULL);
     if ((state && atoi(state)) || !state)
     {
@@ -698,7 +769,7 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     ghex_window_set_doc_name(win, win->gh->document->path_end);
     ghex_window_set_sensitivity(win);
 
-    gtk_signal_emit_by_name(GTK_OBJECT(gh), "cursor_moved");
+    g_signal_emit_by_name(G_OBJECT(gh), "cursor_moved");
    
     return TRUE;
 }
@@ -823,7 +894,7 @@ ghex_window_add_doc_to_list(GHexWindow *win, HexDocument *doc)
 {
     gchar *menu = NULL, *verb_name;
     gchar *cmd = NULL;
-    gchar *escaped_name, *encoded_name;
+    gchar *escaped_name;
     gchar *tip;
 
     escaped_name = encode_xml (doc->path_end);
@@ -869,7 +940,7 @@ void
 ghex_window_set_doc_name(GHexWindow *win, const gchar *name)
 {
     if(name != NULL) {
-        gchar *title = g_strdup_printf(_("GHex - %s"), name);
+        gchar *title = g_strdup_printf(_("%s - GHex"), name);
         gtk_window_set_title(GTK_WINDOW(win), title);
         g_free(title);
     }
@@ -889,7 +960,7 @@ remove_timeout_cb (GtkWidget *win, MessageInfo *mi )
 {
 	g_return_if_fail (mi != NULL);
 
-	gtk_timeout_remove (mi->timeoutid);
+	g_source_remove (mi->timeoutid);
 	g_free (mi);
 }
 
@@ -900,7 +971,7 @@ remove_message_timeout (MessageInfo * mi)
 
 	/* Remove the status message */
 	/* NOTE : Use space ' ' not an empty string '' */
-	ghex_window_show_status (mi->win, " ");
+	ghex_window_update_status_message (mi->win);
     g_signal_handlers_disconnect_by_func(G_OBJECT(mi->win),
                                          remove_timeout_cb, mi);
 	g_free (mi);
@@ -951,20 +1022,19 @@ ghex_window_flash (GHexWindow * win, const gchar * flash)
 	g_return_if_fail (GHEX_IS_WINDOW (win));
 	g_return_if_fail (flash != NULL);
 
-
 	mi = g_new (MessageInfo, 1);
 
 	ghex_window_show_status (win, flash);
 
 	mi->timeoutid =
-		gtk_timeout_add (flash_length,
-                         (GtkFunction) remove_message_timeout,
-                         mi);
+		g_timeout_add (flash_length,
+                       (GSourceFunc) remove_message_timeout,
+                       mi);
 	mi->handlerid =
-		gtk_signal_connect (GTK_OBJECT(win),
-                            "destroy",
-                            GTK_SIGNAL_FUNC (remove_timeout_cb),
-                            mi );
+		g_signal_connect (G_OBJECT(win),
+                          "destroy",
+                          G_CALLBACK (remove_timeout_cb),
+                          mi );
 	mi->win = win;
 }
 
@@ -989,85 +1059,81 @@ ghex_window_save_as(GHexWindow *win)
 {
 	HexDocument *doc;
 	GtkWidget *file_sel;
-	gboolean resp, ret_val = TRUE;
+	gboolean ret_val = TRUE;
 	const gchar *filename;
 	gboolean dir_flag = FALSE;
+    GtkResponseType resp;
 
 	if(win->gh == NULL)
 		return ret_val;
 
 	doc = win->gh->document;
-	file_sel = gtk_file_selection_new(NULL);
+	file_sel =
+        gtk_file_chooser_dialog_new(_("Select a file to save buffer as"),
+                                    GTK_WINDOW(win),
+                                    GTK_FILE_CHOOSER_ACTION_SAVE,
+                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                    GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+                                    NULL);
 
 	if(doc->file_name)
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel),
-										doc->file_name);
-
-	gtk_window_set_title(GTK_WINDOW(file_sel), _("Select a file to save buffer as"));
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_ok_cb),
-						&resp);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-						"clicked", GTK_SIGNAL_FUNC(file_sel_cancel_cb),
-						&resp);
-	gtk_signal_connect (GTK_OBJECT (file_sel),
-						"delete-event", GTK_SIGNAL_FUNC(file_sel_delete_event_cb),
-						&resp);
-
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(file_sel),
+                                      doc->file_name);
 	gtk_window_set_modal(GTK_WINDOW(file_sel), TRUE);
-	gtk_window_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
+	gtk_window_set_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (file_sel);
 
 	do {
-		gtk_main();
-		filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
-		if (g_file_test(filename,G_FILE_TEST_IS_DIR)) {
-			gint name_len;
-			gchar *dir_name;
+        resp = gtk_dialog_run(GTK_DIALOG(file_sel));
+        if(resp == GTK_RESPONSE_OK) {
+            filename =
+                gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_sel));
+            if (g_file_test(filename,G_FILE_TEST_IS_DIR)) {
+                gint name_len;
+                gchar *dir_name;
 
-			name_len = strlen (filename);
-			if (name_len < 1 || filename [name_len - 1] != '/')	{
-				dir_name = g_strconcat (filename, "/", NULL);
-			} else  {
-				dir_name = g_strdup (filename);
-			}
-			gtk_file_selection_set_filename (file_sel, dir_name);
-			g_free (dir_name);
-			dir_flag = TRUE;
-		}
-		else  {
-	     dir_flag = FALSE;
-		}
-	} while (resp && dir_flag);	   
+                name_len = strlen (filename);
+                if (name_len < 1 || filename [name_len - 1] != '/')	{
+                    dir_name = g_strconcat (filename, "/", NULL);
+                } else  {
+                    dir_name = g_strdup (filename);
+                }
+                gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(file_sel),
+                                               dir_name);
+                g_free (dir_name);
+                dir_flag = TRUE;
+            }
+            else  {
+                dir_flag = FALSE;
+            }
+        }
+	} while (resp == GTK_RESPONSE_OK && dir_flag);	   
 
-
-
-	if(resp) {
+	if(resp == GTK_RESPONSE_OK) {
 		FILE *file;
 		gchar *flash;
 		int i;
-        gint save = 0;
         gchar *gtk_file_name, *path_end;
 
         if(access(filename, F_OK) == 0) {
-            GnomeMessageBox *mbox;
-            gchar *msg;
+            GtkWidget *mbox;
 
             gtk_file_name = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
-            msg= g_strdup_printf(_("File %s exists.\n"
-                                   "Do you want to overwrite it?"),
-                                 gtk_file_name);
-            g_free(gtk_file_name);
-            mbox = GNOME_MESSAGE_BOX(gnome_message_box_new(msg,
-                                                           GNOME_MESSAGE_BOX_QUESTION,
-                                                           GNOME_STOCK_BUTTON_YES,
-                                                           GNOME_STOCK_BUTTON_NO,
-                                                           NULL));
-			 gtk_window_set_type_hint (GTK_WINDOW (mbox), GDK_WINDOW_TYPE_HINT_DIALOG);
-			 gtk_window_set_transient_for (GTK_WINDOW (mbox), GTK_WINDOW (file_sel));
 
-            g_free(msg);
-            ret_val = (ask_user(mbox) == 0);
+			mbox = gtk_message_dialog_new(GTK_WINDOW(win),
+										  GTK_DIALOG_MODAL|
+										  GTK_DIALOG_DESTROY_WITH_PARENT,
+										  GTK_MESSAGE_QUESTION,
+										  GTK_BUTTONS_YES_NO,
+                                          _("File %s exists.\n"
+                                            "Do you want to overwrite it?"),
+                                          gtk_file_name);
+            g_free(gtk_file_name);
+
+            gtk_dialog_set_default_response(GTK_DIALOG(mbox), GTK_RESPONSE_NO);
+
+            ret_val = (ask_user(GTK_MESSAGE_DIALOG(mbox)) == GTK_RESPONSE_YES);
+            gtk_widget_destroy (mbox);
         }
 
         if(ret_val) {
@@ -1077,10 +1143,10 @@ ghex_window_save_as(GHexWindow *win)
                         g_free(doc->file_name);
                     if(doc->path_end)
                         g_free(doc->path_end);
-                    doc->file_name = strdup(filename);
+                    doc->file_name = g_strdup(filename);
                     doc->changed = FALSE;
                     win->changed = FALSE;
-                    
+
                     for(i = strlen(doc->file_name);
                         (i >= 0) && (doc->file_name[i] != '/');
                         i--)
@@ -1131,7 +1197,6 @@ ghex_window_uri_exists (const gchar* text_uri)
 gboolean
 ghex_window_ok_to_close(GHexWindow *win)
 {
-	static char msg[MESSAGE_LEN + 1];
 	GtkWidget *mbox;
 	gint reply;
 	gboolean uri_exists;
@@ -1146,16 +1211,13 @@ ghex_window_ok_to_close(GHexWindow *win)
     if(!hex_document_has_changed(doc) && uri_exists)
         return TRUE;
 
-	g_snprintf(msg, MESSAGE_LEN,
-			   _("File %s has changed since last save.\n"
-				 "Do you want to save changes?"),
-			   doc->path_end);
-
 	mbox = gtk_message_dialog_new(GTK_WINDOW(win),
 								  GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
 								  GTK_MESSAGE_QUESTION,
 								  GTK_BUTTONS_NONE,
-								  msg);
+                                  _("File %s has changed since last save.\n"
+                                    "Do you want to save changes?"),
+                                  doc->path_end);
 			
 	save_btn = create_button(mbox, GTK_STOCK_NO, _("Do_n't save"));
 	gtk_widget_show (save_btn);
