@@ -29,8 +29,12 @@
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 
-#include "ghex.h"
+#include "ui.h"
 #include "ghex-window.h"
+#include "findreplace.h"
+#include "converter.h"
+#include "print.h"
+#include "chartable.h"
 
 static void ghex_print(GtkHex *gh, gboolean preview);
 static void ghex_print_run_dialog(GHexPrintJobInfo *pji);
@@ -40,22 +44,38 @@ static void ghex_print_document_real (GHexPrintJobInfo *pji, gboolean preview);
 /* callbacks to nullify widget pointer after a delete event */
 static void about_destroy_cb(GtkObject *, GtkWidget **);
 
-static void open_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void save_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void save_as_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void print_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void print_preview_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void cut_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void copy_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void paste_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void help_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void DnDNewWindow_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname);
-static void DnDCancel_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname);
+static void open_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void close_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void save_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void save_as_cb (BonoboUIComponent *uic, gpointer user_data,
+						const gchar* verbname);
+static void print_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void print_preview_cb (BonoboUIComponent *uic, gpointer user_data,
+							  const gchar* verbname);
+static void export_html_cb (BonoboUIComponent *uic, gpointer user_data,
+							const gchar* verbname);
+static void revert_cb (BonoboUIComponent *uic, gpointer user_data,
+					   const gchar* verbname);
+static void prefs_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void about_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void cut_cb (BonoboUIComponent *uic, gpointer user_data,
+					const gchar* verbname);
+static void copy_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void paste_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void help_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void DnDNewWindow_cb(BonoboUIComponent *uic, gpointer user_data,
+							const char *cname);
+static void DnDCancel_cb(BonoboUIComponent *uic, gpointer user_data,
+						 const char *cname);
 
 guint group_type[3] = {
 	GROUP_BYTE,
@@ -213,15 +233,15 @@ about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	static GtkWidget *about = NULL;
 	
 	static const gchar *authors[] = {
-		"Jaka Mocnik <jaka@gnu.org>",
-		"Chema Celorio <chema@celorio.com>",
-		"Shivram U <shivaram.upadhyayula@wipro.com>",
+		"Jaka Mo\304\215nik",
+		"Chema Celorio",
+		"Shivram Upadhyayula",
 		NULL
 	};
 
 	if(!about) {
 		about = gnome_about_new ( _("GHex, a binary file editor"), VERSION,
-								  "(C) 1998 - 2004 Jaka Mocnik",
+								  "Copyright 1998 - 2006 Jaka Mo\304\215nik",
 								  _("Released under the terms of GNU Public License"),
 								  authors, NULL, NULL, NULL);
 		g_signal_connect(G_OBJECT(about), "destroy",
@@ -264,7 +284,7 @@ paste_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_paste_clipboard(win->gh);
+		gtk_hex_paste_from_clipboard(win->gh);
 }
 
 void 
@@ -273,7 +293,7 @@ copy_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_copy_clipboard(win->gh);
+		gtk_hex_copy_to_clipboard(win->gh);
 }
 
 void 
@@ -282,7 +302,7 @@ cut_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_cut_clipboard(win->gh);
+		gtk_hex_cut_to_clipboard(win->gh);
 }
 
 void
@@ -577,6 +597,10 @@ close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 
     /* this implicitly destroys all views including this one */
     g_object_unref(G_OBJECT(doc));
+
+    /* This is to clear the contents of status  bar after closing the files */
+    bonobo_ui_component_set_status(win->uic, " ", NULL);
+
 }
 
 void
@@ -616,7 +640,7 @@ void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname
 	if(!prefs_ui)
 		prefs_ui = create_prefs_dialog();
 
-	set_prefs(prefs_ui);
+	set_current_prefs(prefs_ui);
 
 	if(ghex_window_get_active() != NULL)
 		gtk_window_set_transient_for(GTK_WINDOW(prefs_ui->pbox),
